# TPREPROC - Run image processing tasks on both TRES amps, then combine them
# September 26, 2007
# By Doug Mink, SAO Telescope Data Center

procedure tpreproc (images)

string	images=""	{prompt="Images to merge"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	imfile, newfile, imlist
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex
bool	vb

	vb = verbose
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("TPREPROC: Run image processing tasks one amp at a time, then combine amps")
	    print ("Usage: tpreproc <input images with both amps to be corrected")
	    return
	    }

	if (!defpac ("mscred"))
	    mscred
	if (!deftask ("ccdproc"))
	    mscred

# Make directory in which to save unprocessed files
	if (saveold) {
	    if (!access ("Raw"))
		mkdir ("Raw")
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("temproclist")
	if (tchar == '@') {
	    imlist = substr (ims,2,100)
	    copy (imlist, templist)
	    }
	else {
	    files (ims, >templist)
	    }

# Read through input files
	list = templist
        while (fscan (list,imin) != EOF) {

# Do not process image twice
	    imgetp.default = "0"
	    imsec = imin // "[0]"
	    imgetp (imsec, "TPREPROC")
#	    imgetp (imin, "TPREPROC")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TPREPROC: ", imin, " has already been processed")
		    print ("TPREPROC = ", imgetp.value)
		    }
		}

# Find root name of image
	    else {

#  Rename original file
		if (saveold) {
		    imr = "Raw/" // imin
		    if (!access (imr))
			copy (imin, imr)
		    }

# Find root name of image file
		imroot = imin
		smatch (imroot, imx)
		if (smatch.found) {
		    imroot = substr (imroot,1,smatch.start-1)
		    }
		else {
		    imroot = imin
		    imin = imin // imx
		    }
#		if (vb) {
#		    print ("TPREPROC: image root is ",imroot)
#		    }

# Make up names for the single-preamplifier images
		amp1 = imroot // "_amp1.fits"
		amp2 = imroot // "_amp2.fits"

# Add trimming information to single-preamp images for CCDPROC
#		hedit (amp1,"DATASEC","[51:1074,3:4608]",up+,ver-,sh+,addon+)
#		hedit (amp1,"TRIMSEC","[51:1074,3:4608]",up+,ver-,sh+,addon+)
#		hedit (amp1,"BIASSEC","[4:46,3:4608]",up+,ver-,sh+,addon+)
#		hedit (amp2,"DATASEC","[17:1040,3:4608]",up+,ver-,sh+,addon+)
#		hedit (amp2,"TRIMSEC","[17:1040,3:4608]",up+,ver-,sh+,addon+)
#		hedit (amp2,"BIASSEC","[1044:1086,3:4608]",up+,ver-,sh+,addon+)

#  Trim, subtract overscan, and fix bad pixels using imred.ccdred.ccdproc
		ccdproc.bpmasks = ""
		ccdproc.ccdtype = ""
		ccdproc.noproc = no
		ccdproc.xtalkcor = no
		ccdproc.fixpix = no
		ccdproc.overscan = yes
		ccdproc.trim = yes
		ccdproc.zerocor = no
		ccdproc.darkcor = no
		ccdproc.flatcor = no
		ccdproc.sflatcor = no
		ccdproc.split = no
		ccdproc.merge = yes
		ccdproc.xtalkfile = ""
		ccdproc.fixfile = ""
		ccdproc.saturation = "INDEF"
		ccdproc.sgrow = 0
		ccdproc.bleed = "INDEF"
		ccdproc.btrail = 20
		ccdproc.bgrow = 0
		ccdproc.biassec = "!BIASSEC"
		ccdproc.trimsec = "!TRIMSEC"
		ccdproc.zero = "ZERO"
		ccdproc.dark = "DARK"
		ccdproc.flat = "FLAT"
		ccdproc.sflat = ""
		ccdproc.minreplace = 1.
		ccdproc.interactive = no
		ccdproc.function = "legendre"
		ccdproc.order = 1
		ccdproc.sample = "*"
		ccdproc.naverage = 1
		ccdproc.niterate = 5
		ccdproc.low_reject = 3.
		ccdproc.high_reject = 3.
		ccdproc.grow = 0.
		ccdproc.fd = ""
		ccdproc.fd2 = ""

#  Trim, subtract overscan, and fix bad pixels using mscred.ccdproc
		temp2 = imroot // "_out.fits"
		ccdproc (imin, output=temp2)
		if (verbose)
		    print ("TPREPROC: fixpix, overscan, and trim ", imin," -> ", temp2)

# Split single input image into two single-preamp images
#		imex = imin // "[1:1090,1:4610]"
#		imex = imin // "[1]"
#		imcopy (imex, amp1)
#		imex = imin // "[1091:2180,1:4610]"
#		imex = imin // "[2]"
#		imcopy (imex, amp2)

#  Adjust second amplifiers to same gain as first one
#		if (gaincorr != 1.0 && gaincorr != 0.0) {
#		    temp3 = mktemp (imroot) // "_amp2g" // imx
#		    if (vb)
#			print ("TPREPROC: ",amp2," gain corrected to ", temp3)
#		    imarith (amp2, "*", gaincorr, temp3)
#		    delete (amp2)
#		    rename (temp3, amp2)
#		    }

#  Recombine two amplifier files into single image file
#		imlist = amp1 // "," // amp2
#		imjoin (imlist, imin)

# Add history line including time
		gettime ()
		history = gettime.date // " " // gettime.time // " fixpix, overscan, and trim"
		hedit (imin, "TPREPROC", history, add+, verify-)
		}
	    }

	if (deltemp)
	    delete (templist)
end

# Sep 26 2007	New program to process list of TRES images
